@@ -3,7 +3,7 @@ use warnings;
 
 package Oh;
 
-our $VERSION = '1.01';
+our $VERSION = '1.02';
 
 use subs qw/error env listp evaluate_element evaluate_list evaluate_function evaluate_builtin evaluate_macro evaluate_lambda find search progn read_element read_string read_list reft add_value lisp_args scheme_args/;
 
@@ -1255,6 +1255,32 @@ op
 } 'until';
 
 
+op
+{
+  my $name = shift;
+  my $macro = find 'macro', $name;
+  if ($macro)
+  {
+    my $type = ref $macro;
+    if ($type eq 'CODE')
+    {
+      evaluate_builtin $macro, @_;
+    }
+    elsif ($type eq 'macro')
+    {
+      evaluate_lambda $macro, @_;
+    }
+    else
+    {
+      error 'wrong macro type', $macro, $type;
+    }
+  }
+  else
+  {
+    error 'macroexpand did not find a macro', $macro;
+  }
+
+} 'macroexpand';
 
 #funs
 
@@ -1622,6 +1648,68 @@ fun
   }
 } 'decr';
        
+fun
+{
+  my $list = shift;
+  push @$list, @_;
+} 'push';
+
+fun
+{
+  my $list = shift;
+  unshift @$list, @_;
+} 'unshift';
+
+fun { shift @{shift()} } 'shift';
+fun { pop @{shift()} } 'pop';
+
+fun
+{
+  my ($list, $start, $end) = @_;
+  if ($end)
+  {
+    [splice @$list, $start, $end];
+  }
+  else
+  {
+    [splice @$list, $start];
+  }
+} 'splice';
+
+fun
+{
+  my ($hash, $key) = @_;
+  delete $hash->{$key};
+} 'remove-key';
+
+fun
+{
+  my $hash = shift;
+  for my $key (@_)
+  {
+    delete $hash->{$key};
+  }
+} 'remove-keys';
+
+fun
+{
+  my $hash = shift;
+  while (@_)
+  {
+    my ($key, $value) = splice @_, 0, 2;
+    $hash->{$key} = $value;
+  }
+} 'set-keys';
+
+fun
+{
+  my $str = '';
+  for my $string (@_)
+  {
+    $str .= $string;
+  }
+  bless \$str, 'string';
+} '.';
 
 #ets
 
